package cn.com.taiji.tile.service.impl;
import cn.com.taiji.common.domain.PageResult;
import cn.com.taiji.cql.model.GeoDataSource;
import cn.com.taiji.cql.model.GeoRequestParam;
import cn.com.taiji.cql.service.IECqlService;
import cn.com.taiji.tile.service.ITileService;
import cn.com.taiji.tile.util.TileUtils;
import lombok.extern.slf4j.Slf4j;
import org.geotools.jdbc.JDBCDataStoreFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
@Service
public class TileServiceImpl implements ITileService {

    @Autowired
    private IECqlService ecqlService;


    @Override
    public byte[] tileData(GeoRequestParam requestParam,GeoDataSource geoDs) {

        //2根据cql规则获取图层对应的空间地理数据


        StringBuffer sbBbox = new StringBuffer();
        sbBbox.append("(bbox (");
        sbBbox.append(geoDs.getBboxColumn()+",");
        sbBbox.append(requestParam.getBbox()+"))");


//        String filterCql = requestParam.getCqlFilter()+" and "+sbBbox.toString();
//        String filterCql = sbBbox.toString();

        String filterCql = requestParam.getCqlFilter();

        System.out.println("查询条件:"+filterCql);

        Map<String, Object> cqlDsParams = new HashMap<String, Object>();
        cqlDsParams.put(JDBCDataStoreFactory.DATABASE.key, geoDs.getDataBase());
        cqlDsParams.put(JDBCDataStoreFactory.DBTYPE.key, geoDs.getDbType());

        cqlDsParams.put(JDBCDataStoreFactory.HOST.key, geoDs.getHost());
        cqlDsParams.put(JDBCDataStoreFactory.PORT.key, geoDs.getPort());
        cqlDsParams.put(JDBCDataStoreFactory.USER.key, geoDs.getUserName());
        cqlDsParams.put(JDBCDataStoreFactory.PASSWD.key, geoDs.getUserPassword());
        cqlDsParams.put("tableName", geoDs.getTableName());

        //组装排序参数
        Map<String, Object> sortParams = new HashMap<String, Object>();
        sortParams.put("sort","asc");

        PageResult pageResult = ecqlService.cqlQuery4Mysql(cqlDsParams,filterCql,sortParams,null);

        if(pageResult.getRecordCount() >0){

            List<Map<String, Object>> cqlDataList = (List<Map<String, Object>>) pageResult.getDatas();

            //3根据空间地理数据生成图层二进制数据
            System.out.println("返回的数据:"+cqlDataList.toString());

            return TileUtils.getTile(cqlDataList, requestParam.getBbox());

        }else{

            log.info("根据cql规则:"+sbBbox.toString()+"和图层代码:" + requestParam.getLayers() + " 没有找到图层对应的数据源的空间地理数据");
        }

        return new byte[0];
    }

    @Override
    public List<?> queryElements(String bbox, Integer type, String layerName) {
        //1. 将bbox转换为四个点位

        return null;
    }

    public static double[] parseBbox(String bboxString) {
        String[] bboxValuesStr = bboxString.split(",");
        if (bboxValuesStr.length != 4) {
            return new double[0]; // 返回空数组表示无效的 BBOX 格式
        }

        double[] bboxValues = new double[4];
        try {
            for (int i = 0; i < 4; i++) {
                bboxValues[i] = Double.parseDouble(bboxValuesStr[i]);
            }
        } catch (NumberFormatException e) {
            return new double[0]; // 返回空数组表示无效的 BBOX 格式
        }

        return bboxValues;
    }

}
